package intermediate;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * The symbol table.
 */
public class SymbolTable 
{
    private final int level;                       // nesting level
    private TreeMap<String, SymtabEntry> entries;  // the entries    
    
    /**
     * Constructor.
     * @param level the nesting level.
     */
    public SymbolTable(int level)
    {
        this.level = level;
        this.entries = new TreeMap<>();
    }
    
    public int getLevel() { return level; }
    
    /**
     * Create the global symbol table.
     * @return the global symbol table.
     */
    public static SymbolTable createGlobal()
    {
        SymbolTable global = new SymbolTable(0);
        
        global.enter("+", SymtabEntryType.BUILT_IN);
        global.enter("*", SymtabEntryType.BUILT_IN);
        
        return global;
    }
    
    /**
     * Enter a symbol into the symbol table.
     * @param symbol the symbol.
     * @param type the symbol type.
     * @return the new entry.
     */
    public SymtabEntry enter(String symbol, SymtabEntryType type)
    {
        SymtabEntry entry = entries.get(symbol);
        
        // Enter only if it's not already in the table.
        if (entry == null) {
            entry = new SymtabEntry(symbol, type, this);
            entries.put(symbol, entry);
            return entry;
        }
        else {
            System.out.printf("*** Redefined symbol '%s'\n", symbol);
            return entry;
        }
    }
    
    /**
     * Find a symbol in this table.
     * @param symbol the symbol to find.
     * @return the symbol's entry if found, else null.
     */
    public SymtabEntry find(String symbol)
    {
        return entries.get(symbol);
    }
    
    /**
     * The entries are automatically sorted alphabetically.
     * @return an iterator for the entries in this table.
     */
    public Iterator<Map.Entry<String, SymtabEntry>> iterator()
    {
        Set<Map.Entry<String, SymtabEntry>> entrySet = entries.entrySet();
        return entrySet.iterator();
    }
    
    /**
     * Print the entries in this table in alphabetical order.
     */
    public void print()
    {
        System.out.printf("\n==== SYMBOL TABLE Level %d ====\n", level);
        
        Iterator<Map.Entry<String, SymtabEntry>> it = iterator();
        
        // Iterate over the alphabetized contents.
        while (it.hasNext()) {
            Map.Entry<String, SymtabEntry> entry = it.next();
            System.out.printf("%8s : %s\n", 
                              entry.getKey(), entry.getValue().getType());
        }

        System.out.printf("==============================\n\n", level);
    }
}
